/* gcjwebplugin - Webbrowser plugin to execute Java (tm) applets.
   Copyright (C) 2003  Michael Koch <konqueror@gmx.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */


#include "jniHelp.h"
#include "pluginDebug.h"
#include "GCJJVMConsole.h"

GCJJVMConsole::GCJJVMConsole (JNIEnv* jniEnv)
  : m_jniEnv (jniEnv)
  , m_isVisible (false)
{
  PLUGIN_DEBUG ("GCJJVMConsole::GCJJVMConsole\n");
  
  // FIXME: Initialize this.
  m_consoleDialog = NULL;
}

GCJJVMConsole::~GCJJVMConsole()
{
  PLUGIN_DEBUG ("GCJJVMConsole::~GCJJVMConsole\n");
}

NS_IMPL_ISUPPORTS1 (GCJJVMConsole, nsIJVMConsole)

NS_IMETHODIMP
GCJJVMConsole::IsVisible (PRBool* result)
{
  PLUGIN_DEBUG ("GCJJVMConsole::IsVisible\n");
  
  (*result) = m_isVisible ? PR_TRUE : PR_FALSE;
  return NS_OK;
}

NS_IMETHODIMP
GCJJVMConsole::Show ()
{
  PLUGIN_DEBUG ("GCJJVMConsole::Show\n");
  
  assert (m_consoleDialog != NULL);

  jclass consoleDialog_cl;
  jmethodID consoleDialog_show;

  JNI_GetObjectClass (m_jniEnv, m_consoleDialog, consoleDialog_cl);
  JNI_GetMethodID (m_jniEnv, consoleDialog_cl, "show", "()V", consoleDialog_show);
  JNI_CallVoidMethod0 (m_jniEnv, m_consoleDialog, consoleDialog_show);
  
  return NS_OK;
}

NS_IMETHODIMP
GCJJVMConsole::Hide ()
{
  PLUGIN_DEBUG ("GCJJVMConsole::Hide\n");
  
  assert (m_consoleDialog != NULL);

  jclass consoleDialog_cl;
  jmethodID consoleDialog_hide;
  
  JNI_GetObjectClass (m_jniEnv, m_consoleDialog, consoleDialog_cl);
  JNI_GetMethodID (m_jniEnv, consoleDialog_cl, "hide", "()V", consoleDialog_hide);
  JNI_CallVoidMethod0 (m_jniEnv, m_consoleDialog, consoleDialog_hide);
  
  return NS_OK;
}

NS_IMETHODIMP
GCJJVMConsole::Print (const char* text, const char* encoding)
{
  PLUGIN_DEBUG ("GCJJVMConsole::Print\n");
  
  assert (m_consoleDialog != NULL);

  jclass consoleDialog_cl;
  jmethodID consoleDialog_print;
 
  JNI_GetObjectClass (m_jniEnv, m_consoleDialog, consoleDialog_cl);
  JNI_GetMethodID (m_jniEnv, consoleDialog_cl, "print",
		   "(Ljava.lang.String;Ljava.lang.String;)V", consoleDialog_print);

  // FIXME: Handle case when encoding is NULL
  jstring jv_text = m_jniEnv->NewStringUTF (text);
  jstring jv_encoding = m_jniEnv->NewStringUTF (encoding);

  JNI_CallVoidMethod2 (m_jniEnv, m_consoleDialog, consoleDialog_print, jv_text, jv_encoding);
  
  return NS_OK;
}
